# -*- coding: utf-8 -*-
"""
Read me:
This file supports the 'main_v4_looper.py' file. It includes data cleaning
and pre-processing steps, as well as the function that estimates Armington
elasticities from trade data.
"""

#######################
### Import packages ###
#######################
import pandas as pd
import numpy as np
import statsmodels.api as sm
import re
import os

#####################################
### Define data-related functions ###
#####################################

def DataWebClean(filename, saveclean=True):
    ### Download data from DataWeb ###
    # 'filename' should include .xlsx
    # Must select 'Customs Value' and 'Landed Duty-Paid Value'.
    # Select any number of years, and choose 'Annual' aggregation.
    # Choose any or all countries, but must 'Display Countries Separately'.
    # Choose commodities and choose aggregation.
    # Must 'Display Commodity Description'.
    # Must 'Aggregate All Import Programs Together'.
    # Must 'Aggregate All Provision Codes Together'.
    # Must 'Display Districts Separately'.
    # DO NOT select 'Display Percentage Change Column'.
    # DO NOT select 'Downloading Data: Combine Rows into One Sheet'.
    
    filename_base, filename_ext = os.path.splitext(filename)
    filenamenew = filename_base + '_clean.xlsx'
    
    # Import Customs Value (cv) data
    data_cv = pd.read_excel(filename,sheet_name='Customs Value',header=2, 
                            converters={'HTS Number':str}, engine='openpyxl')
    datalong_cv = pd.melt(data_cv.loc[data_cv['Data Type']!='Total:'],id_vars=['Data Type','Country','HTS Number','Description','District'])
    datalong_cv.rename({'variable':'Year','value':'cv','HTS Number':'HTS'},axis=1,inplace=True)
    datalong_cv['Year'] = datalong_cv['Year'].str[5:]
    datalong_cv.drop('Data Type',axis=1,inplace=True)

    # Import Landed Duty-Paid Value (ldpv) and merge both
    data_ldpv = pd.read_excel(filename,sheet_name='Landed Duty-Paid Value',header=2, 
                              converters={'HTS Number':str}, engine='openpyxl')
    datalong_ldpv = pd.melt(data_ldpv.loc[data_ldpv['Data Type']!='Total:'],id_vars=['Data Type','Country','HTS Number','Description','District'])
    datalong_ldpv.rename({'variable':'Year','value':'ldpv','HTS Number':'HTS'},axis=1,inplace=True)
    datalong_ldpv['Year'] = datalong_ldpv['Year'].str[5:]
    datalong_ldpv.drop('Data Type',axis=1,inplace=True)
    
    # Combine into one dataframe
    datalong = datalong_cv.merge(datalong_ldpv,how='outer',on=['Country','District','Year','HTS','Description'])
    datalong['HTS']=datalong['HTS'].str.replace('[^\w\s]','')
    
    # Remove empty (cv and ldpv are zero) rows
    datalong = datalong.loc[(datalong['cv']>0)|(datalong['ldpv']>0)]
    
    if saveclean==True:
        datalong.to_excel(filenamenew,index=False)
    
    return datalong

def RRupdater(filename):
    ### This incorporates revisions to HTS import values that are not reflected
    #   in the DataWeb data pull.
    df_update_data = pd.read_excel(filename,sheet_name='By District', 
                            converters={'HS':str,'Stat Year':str}, engine='openpyxl')
    df_update_data = df_update_data.loc[df_update_data['Label'].notnull()]
    df_old = df_update_data.loc[df_update_data['Label']=='Reads',['Stat Year','Stat Month','HS','District.1','Value','CIF Value']]
    df_new = df_update_data.loc[df_update_data['Label']=='Should Read',['Stat Year','Stat Month','HS','District.1','Value','CIF Value']]
    df_updater = df_old.merge(df_new,how='inner',on=['Stat Year','Stat Month','HS','District.1'],suffixes=['_old','_new']).reset_index()
    df_updater = df_updater.groupby(['Stat Year','HS','District.1']).agg('sum').reset_index()
    df_updater['cvfrac']=df_updater['Value_new']/df_updater['Value_old']
    df_updater['ldpvfrac']=df_updater['CIF Value_new']/df_updater['CIF Value_old']
    df_updater=df_updater.rename({'Stat Year':'Year','HS':'HTS','District.1':'District'},axis=1)
    df_updater=df_updater[['Year','HTS','District','cvfrac','ldpvfrac']]
    
    df_adder = df_update_data.loc[df_update_data['Label']=='Transferred to',['Stat Year','HS','District.1','Value','CIF Value']]
    df_adder = df_adder.groupby(['Stat Year','HS','District.1']).agg('sum').reset_index()
    df_adder = df_adder.rename({'Stat Year':'Year','HS':'HTS','District.1':'District'},axis=1)
    return df_updater, df_adder

def RRDataWebClean2(filename):
    ## Setup
    # Call DWC to get cv and ldpv in their respective columns
    df_full = DataWebClean(filename,saveclean=False)
    
    # Create new columns for cv,ldpv_IQF and cv,ldpv_Other
    newcols = ['cv_IQF','ldpv_IQF','cv_Other','ldpv_Other']
    for newcol in newcols:
        df_full[newcol] = 0
    
    df_updater, df_adder = RRupdater(
            'raspberries_census_revisions_by_district.xlsx')
    df_full = df_full.merge(df_updater,how='left',on=['HTS','Year','District'])
    df_full.loc[df_full['cvfrac'].isnull(),'cvfrac']=1
    df_full.loc[df_full['ldpvfrac'].isnull(),'ldpvfrac']=1
    
    df_full['cv']=df_full['cv']*df_full['cvfrac']
    df_full['ldpv']=df_full['ldpv']*df_full['ldpvfrac']
    df_full = df_full.drop(['cvfrac','ldpvfrac'],axis=1)
    
    df_addto = df_full[['HTS','Year','District','cv','ldpv']].groupby(['HTS','Year','District']).agg('sum').reset_index()
    df_addto = df_addto.merge(df_adder,how='inner',on=['HTS','Year','District'])
    df_addto['cv_new'] = df_addto['cv']+df_addto['Value']
    df_addto['ldpv_new'] = df_addto['ldpv']+df_addto['CIF Value']
    df_addto['cvfrac']=df_addto['cv_new']/df_addto['cv']
    df_addto['ldpvfrac']=df_addto['ldpv_new']/df_addto['ldpv']    
    df_addto = df_addto[['HTS','Year','District','cvfrac','ldpvfrac']]
    
    df_full = df_full.merge(df_addto,how='left',on=['HTS','Year','District'])
    df_full.loc[df_full['cvfrac'].isnull(),'cvfrac']=1
    df_full.loc[df_full['ldpvfrac'].isnull(),'ldpvfrac']=1
    
    df_full['cv']=df_full['cv']*df_full['cvfrac']
    df_full['ldpv']=df_full['ldpv']*df_full['ldpvfrac']
    df_full = df_full.drop(['cvfrac','ldpvfrac'],axis=1)
    
    ### -----------------------------------------------------------------------
    ### -----------------------------------------------------------------------
    ### -----------------------------------------------------------------------
    
    ## Sorting bulk from fresh for consumption
    # List of partials and in/out for splitting
    # The -22 and -24 versions are only in recent years (precision added to HTS)
    partiallist = ['0810201020','0810209020']
    inlist = ['0810201024','0810209024']
    outlist = ['0810201022','0810209022']
    
    # Put in and out next to each other.
    # Group at HS8 level (basically in-season vs off-season)
    df_full['HS8']=df_full['HTS'].str[0:8]
    df_in = df_full.loc[df_full['HTS'].isin(inlist),['HS8','cv','ldpv']].groupby(['HS8']).agg('sum').reset_index()
    df_out = df_full.loc[df_full['HTS'].isin(outlist),['HS8','cv','ldpv']].groupby(['HS8']).agg('sum').reset_index()
    df_inout = df_in.merge(df_out,on=['HS8'],suffixes=['_in','_out'])

    # New df to calculate the fraction in and out
    splitter = df_inout[['HS8','cv_in','cv_out','ldpv_in','ldpv_out']].groupby('HS8').agg('sum')
    splitter['cvinfrac'] = splitter['cv_in']/(splitter['cv_in']+splitter['cv_out'])
    splitter['ldpvinfrac'] = splitter['ldpv_in']/(splitter['ldpv_in']+splitter['ldpv_out'])
    splitter.drop(['cv_in','cv_out','ldpv_in','ldpv_out'],axis=1,inplace=True)
    
    # Merge splitter dataframe onto main dataframe and use that to calculate
    # how much is in-scope (specifically, Other) vs out-of-scope
    df_full = df_full.merge(splitter,on='HS8',how='left')
    df_full.loc[df_full['HTS'].isin(partiallist),['cv_Other']]=df_full['cv']*df_full['cvinfrac']
    df_full.loc[df_full['HTS'].isin(partiallist),['ldpv_Other']]=df_full['ldpv']*df_full['ldpvinfrac']
    
    # Put the in-scope HTS codes in the correct column
    df_full.loc[df_full['HTS'].isin(inlist),['cv_Other']]=df_full['cv']
    df_full.loc[df_full['HTS'].isin(inlist),['ldpv_Other']]=df_full['ldpv']
    df_full = df_full.drop(['cvinfrac','ldpvinfrac'],axis=1)
    
    ## Now handle mixed bags -> partial IQF, partial OoS
    # Assume 8085 is partially in-scope and 8095 OoS
    # Calculate fraction of 8080 likely to be 8085 vs 8095 and reduce it accordingly.
    # Then use the 7/24 (avg of 1/4 and 1/3) for 8085 and the reduced 8080
    df_hi = df_full.loc[(df_full['HTS']=='0811908085') & (df_full['cv']!=0),['HS8','cv','ldpv']].groupby(['HS8']).agg('sum').reset_index()
    df_lo = df_full.loc[(df_full['HTS']=='0811908095') & (df_full['cv']!=0),['HS8','cv','ldpv']].groupby(['HS8']).agg('sum').reset_index()
    df_hilo = df_hi.merge(df_lo,on=['HS8'],suffixes=['_hi','_lo'])
    df_hilo['cvhifrac']=df_hilo['cv_hi']/(df_hilo['cv_hi']+df_hilo['cv_lo'])
    df_hilo['ldpvhifrac']=df_hilo['ldpv_hi']/(df_hilo['ldpv_hi']+df_hilo['ldpv_lo'])
    df_hilo=df_hilo[['HS8','cvhifrac','ldpvhifrac']]
    
    # Merge back into df_full. 8080 gets multiplied by hifrac.
    # Then 8080 and 8085 are partially put into IQF
    mixfrac = 7/24  # average of 1/4 and 1/3 assumed IQF from mixed berry bags
    df_full = df_full.merge(df_hilo,how='left',on=['HS8'])
    df_full.loc[df_full['HTS']=='0811908080','cv_IQF'] = df_full['cv']*df_full['cvhifrac']*mixfrac
    df_full.loc[df_full['HTS']=='0811908080','ldpv_IQF'] = df_full['ldpv']*df_full['ldpvhifrac']*mixfrac
    df_full.loc[df_full['HTS']=='0811908085','cv_IQF'] = df_full['cv']*mixfrac
    df_full.loc[df_full['HTS']=='0811908085','ldpv_IQF'] = df_full['ldpv']*mixfrac
    
    df_full = df_full.drop(['cvhifrac','ldpvhifrac'],axis=1)
    
     ## 0811.20.2025 is partially IQF, partially Other. 
    # Sorted by country based on research by analysts.
    # First, set default placeholder for when no information for a country
    # is provided. 
    frozen_IQF_default = 0.6
    # When information is given, set IQF percentage for each country.
    frozen_IQF_dict = {
        # CHECK NAMES FOR SPELLING TO MATCH THE DATA
        'Canada':0.4,
        'Chile':0.8,
        'Mexico':0.5,
        'Serbia':0.75
        } 
    # Create new column with this info
    df_full['frac_IQF'] = df_full['Country'].map(frozen_IQF_dict)
    df_full['frac_IQF']=df_full['frac_IQF'].fillna(value=frozen_IQF_default)
    
    df_full.loc[df_full['HTS']=='0811202025','cv_IQF']=df_full['cv']*df_full['frac_IQF']
    df_full.loc[df_full['HTS']=='0811202025','ldpv_IQF']=df_full['ldpv']*df_full['frac_IQF']
    df_full.loc[df_full['HTS']=='0811202025','cv_Other']=df_full['cv']*(1-df_full['frac_IQF'])
    df_full.loc[df_full['HTS']=='0811202025','ldpv_Other']=df_full['ldpv']*(1-df_full['frac_IQF'])
    
    df_full = df_full.drop(['frac_IQF','HS8'],axis=1)
    
    ## Handle juice
    df_full.loc[(df_full['HTS']=='2009896055')|(df_full['HTS']=='2009897055'),'cv_Other']=df_full['cv']
    df_full.loc[(df_full['HTS']=='2009896055')|(df_full['HTS']=='2009897055'),'ldpv_Other']=df_full['ldpv']

    ## Handle cooked
    df_full.loc[(df_full['HTS']=='2007996510'),'cv_Other']=df_full['cv']
    df_full.loc[(df_full['HTS']=='2007996510'),'ldpv_Other']=df_full['ldpv']

    ## Handle other preparations
    df_full.loc[(df_full['HTS']=='2008992120'),'cv_Other']=df_full['cv']
    df_full.loc[(df_full['HTS']=='2008992120'),'ldpv_Other']=df_full['ldpv']
    
    ## Remainder must be out of scope
    df_full['cv_OOS'] = df_full['cv']-df_full['cv_IQF']-df_full['cv_Other']
    df_full['ldpv_OOS'] = df_full['ldpv']-df_full['ldpv_IQF']-df_full['ldpv_Other']    

    ## Rounding errors can make OOS look negative. Change to 0.00
    df_full.loc[(df_full['cv_OOS']<0)&(df_full['cv_OOS']>-1),'cv_OOS']=0.00
    df_full.loc[(df_full['ldpv_OOS']<0)&(df_full['ldpv_OOS']>-1),'ldpv_OOS']=0.00

    return df_full

def RRDomesticClean(filename):
    ### Put the domestic data in similar aggregate groups to the import data
    data = pd.read_excel(filename,sheet_name='Prices and values',
                         header=1,engine='openpyxl',
                         usecols="L,R:T",nrows=11)
    data.columns = ['Year','IQF','S-Pack','Juice']
    df_domestic = pd.melt(data,id_vars='Year').rename({'variable':'Type','value':'Revenue'},axis=1)
    return df_domestic

def RRAggregator3(df_import,df_domestic):
    ### Put domestic and import data in the same file ###
    # Use ldpv for import value (includes tariff revenue)
    
    # IQF first
    df_agg = df_import[['Year','ldpv_IQF']].groupby(['Year']).agg('sum').reset_index()
    df_agg = df_agg.merge(df_domestic.loc[df_domestic['Type']=='IQF',['Year','Revenue']],how='left',on='Year')
    df_agg = df_agg.rename({'ldpv_IQF':'Value_import','Revenue':'Value_domestic'},axis=1)
    df_agg['IQF']=True
    
    # Now Other  
    df_agg = df_agg.append(df_import[['Year','ldpv_Other']].groupby(['Year']).agg('sum').reset_index().rename({'ldpv_Other':'Value_import'},axis=1),sort=False)
    df_agg = df_agg.merge(df_domestic.loc[df_domestic['Type']!='IQF',['Year','Revenue']].groupby('Year').agg('sum').reset_index(),how='left',on='Year')
    
    df_agg = df_agg.rename({'ldpv_IQF':'Import_IQF','ldpv_Other':'Import_Other',
                            'Revenue_IQF':'Domestic_IQF','Revenue_Other':'Domestic_Other'},axis=1)
    df_agg.loc[df_agg['Value_domestic'].isnull(),'Value_domestic']=df_agg['Revenue']
    df_agg.loc[df_agg['IQF'].isnull(),'IQF']=False
    df_agg = df_agg.drop('Revenue',axis=1)

    df_agg['Total']=df_agg['Value_import']+df_agg['Value_domestic']
    df_agg['share_domestic'] = df_agg['Value_domestic']/df_agg['Total']
    
    return df_agg

def ee6(df_import):
    #######################################
    ### Estimate Armington elasticities ###
    #######################################
    
    # Takes cleaned DataWeb df as an input
    datalong = df_import.copy()
    
    # Aggregate by year, country, and district for IQF and Other
    df_agg_ycd = datalong[['Year','Country','District','cv','ldpv']].groupby(['Year','Country','District']).agg(sum)
    df_agg_ycd = df_agg_ycd.reset_index()
    
    # Drop zeros and calculate logged values
    df_agg_ycd = df_agg_ycd.loc[(df_agg_ycd['cv']!=0) & (df_agg_ycd['ldpv']!=0)]
    df_agg_ycd['ln_ldpv']=np.log(df_agg_ycd['cv'])
    df_agg_ycd['ln_ldprat']=np.log(df_agg_ycd['ldpv']/df_agg_ycd['cv'])

    # Create some dummy variables
    df_agg_ycd['cy'] = df_agg_ycd['Country']+str(df_agg_ycd['Year'])
    df_agg_ycd['dy'] = datalong['District']+str(df_agg_ycd['Year'])
    df_agg_ycd['cons_'] = 1
    df_agg_ycd = pd.get_dummies(df_agg_ycd, columns=['cy','dy'],drop_first=True)    
    
    var_list = list(df_agg_ycd.columns)
    X_list = ['ln_ldprat']
    regex = re.compile('cy*')
    cy_list = [word for word in var_list if re.match(regex,word)]
    regex = re.compile('dy*')
    dy_list = [word for word in var_list if re.match(regex,word)]
    X_list.extend(cy_list)
    X_list.extend(dy_list)

    y = df_agg_ycd['ln_ldpv'].copy()
    X = df_agg_ycd[X_list].copy()
    model = sm.OLS(y,X)
    results = model.fit()
    return results.params[0],results.bse[0]