# -*- coding: utf-8 -*-
"""
These helper functions are used in the main file.
"""
import pandas as pd
import numpy as np
from scipy.stats.mstats import gmean

import pype_perfect as PyPE


def simulation_imports(df,year_list,Qimports_IQF,surge_IQF,Qimports_Other,surge_Other,partial=1.0):
    # df is the dataframe to which columns will be added
    # year_list is the period of analysis, 2010-2019
    # Qimports_IQF and Qimports_Other are the value of imports in list form, 2010-2019
    # surge_IQF and surge_Other are the indices of the years identified as surge years
    # e.g. surge_IQF = [2,3] means IQF surge years were 2012-2013
    
    T = len(year_list)
    
    ### IQF ###
    
    # Calculate percentage changes for IQF imports value
    pcdiff_IQF = np.divide(
        np.subtract(Qimports_IQF[1:],Qimports_IQF[:-1]),
        Qimports_IQF[:-1]
        )
    # Calculate surge and non-surge averages
    pcdiff_IQF_avg_hi = np.mean([pcdiff_IQF[i] for i in range(T-1) if i in surge_IQF])
    pcdiff_IQF_avg_lo = np.mean([pcdiff_IQF[i] for i in range(T-1) if i not in surge_IQF])
    # Create full sequence with non-surge average
    pcdiff_IQF_cf = [val-partial*(pcdiff_IQF_avg_hi+pcdiff_IQF_avg_lo) 
                          if i in surge_IQF
                          else val for i,val in enumerate(pcdiff_IQF)]
    
    ### Other ###
    
    # Calculate percentage changes for non-IQF imports value
    pcdiff_Other = np.divide(
        np.subtract(Qimports_Other[1:],Qimports_Other[:-1]),
        Qimports_Other[:-1]
        )
    # Calculate surge and non-surge averages
    pcdiff_Other_avg_hi = np.mean([pcdiff_Other[i] for i in range(T-1) if i in surge_Other])
    pcdiff_Other_avg_lo = np.mean([pcdiff_Other[i] for i in range(T-1) if i not in surge_Other])
    # Create full sequence with non-surge average
    pcdiff_Other_cf = [val-partial*(pcdiff_Other_avg_hi+pcdiff_Other_avg_lo) 
                          if i in surge_Other
                          else val for i,val in enumerate(pcdiff_Other)]
    
    # Use percentage change sequence to create sequence levels
    Qimports_IQF_cf = [Qimports_IQF[0]]
    Qimports_Other_cf = [Qimports_Other[0]]
    for i in range(len(pcdiff_IQF_cf)):
        Qimports_IQF_cf.append(Qimports_IQF_cf[i]*(1+pcdiff_IQF_cf[i]))
        Qimports_Other_cf.append(Qimports_Other_cf[i]*(1+pcdiff_Other_cf[i]))
    
    df_tmpA = pd.DataFrame([year_list,Qimports_IQF_cf]).transpose().rename({0:'Year',1:'Simulation Imports'},axis=1)
    df_tmpA['IQF']=True
    df_tmpB = pd.DataFrame([year_list,Qimports_Other_cf]).transpose().rename({0:'Year',1:'Simulation Imports'},axis=1)
    df_tmpB['IQF']=False
    
    df_tmp = df_tmpA.append(df_tmpB,sort=False)
    
    return df.merge(df_tmp,how='left',on=['Year','IQF'])

def simulation_imports_geo(df,year_list,Qimports_IQF,surge_IQF,Qimports_Other,surge_Other,partial=1.0):
    # df is the dataframe to which columns will be added
    # year_list is the period of analysis, 2010-2019
    # Qimports_IQF and Qimports_Other are the value of imports in list form, 2010-2019
    # surge_IQF and surge_Other are the indices of the years identified as surge years
    # e.g. surge_IQF = [2,3] means IQF surge years were 2012-2013
    
    T = len(year_list)
    
    ### IQF ###
    
    # Calculate percentage changes for IQF imports value
    pcdiff_IQF = np.divide(
        Qimports_IQF[1:],
        Qimports_IQF[:-1]
        )
    # Calculate surge and non-surge averages
    pcdiff_IQF_avg_hi = gmean([pcdiff_IQF[i] for i in range(T-1) if i in surge_IQF])
    pcdiff_IQF_avg_lo = gmean([pcdiff_IQF[i] for i in range(T-1) if i not in surge_IQF])
    # Create full sequence with non-surge average
    pcdiff_IQF_cf = [partial*val*(pcdiff_IQF_avg_lo/pcdiff_IQF_avg_hi)
                        +(1.0-partial)*val
                          if i in surge_IQF
                          else val for i,val in enumerate(pcdiff_IQF)]
    
    ### Other ###
    
    # Calculate percentage changes for non-IQF imports value
    pcdiff_Other = np.divide(
        Qimports_Other[1:],
        Qimports_Other[:-1]
        )
    # Calculate surge and non-surge averages
    pcdiff_Other_avg_hi = gmean([pcdiff_Other[i] for i in range(T-1) if i in surge_Other])
    pcdiff_Other_avg_lo = gmean([pcdiff_Other[i] for i in range(T-1) if i not in surge_Other])
    # Create full sequence with non-surge average
    pcdiff_Other_cf = [partial*val*(pcdiff_Other_avg_lo/pcdiff_Other_avg_hi)
                        +(1.0-partial)*val 
                          if i in surge_Other
                          else val for i,val in enumerate(pcdiff_Other)]
    
    # Use percentage change sequence to create sequence levels
    Qimports_IQF_cf = [Qimports_IQF[0]]
    Qimports_Other_cf = [Qimports_Other[0]]
    for i in range(len(pcdiff_IQF_cf)):
        Qimports_IQF_cf.append(Qimports_IQF_cf[i]*(pcdiff_IQF_cf[i]))
        Qimports_Other_cf.append(Qimports_Other_cf[i]*(pcdiff_Other_cf[i]))
    
    df_tmpA = pd.DataFrame([year_list,Qimports_IQF_cf]).transpose().rename({0:'Year',1:'Simulation Imports'},axis=1)
    df_tmpA['IQF']=True
    df_tmpB = pd.DataFrame([year_list,Qimports_Other_cf]).transpose().rename({0:'Year',1:'Simulation Imports'},axis=1)
    df_tmpB['IQF']=False
    
    df_tmp = df_tmpA.append(df_tmpB,sort=False)
    
    return df.merge(df_tmp,how='left',on=['Year','IQF'])

def include_supply_param(df,multiplier,elasticity):    
    df['Supply Param A'] = 0.0
    df.loc[df['IQF']==True,['Supply Param A']] = multiplier*df['Value_domestic']
    df.loc[df['IQF']==False,['Supply Param A']] = elasticity
    return df

def include_elasticities(df,arm_IQF_dist,arm_Other_dist,
                         elasticity_total_IQF,elasticity_total_Other):
    df_tmpA = pd.DataFrame(arm_IQF_dist,columns=['Armington'])
    df_tmpA['IQF']=True
    df_tmpA['Total Elasticity']=elasticity_total_IQF
    
    df_tmpB = pd.DataFrame(arm_Other_dist,columns=['Armington'])
    df_tmpB['IQF']=False
    df_tmpB['Total Elasticity']=elasticity_total_Other
    
    df_tmp = df_tmpA.append(df_tmpB,sort=False)
    
    return df.merge(df_tmp,how='outer',on='IQF')

def calibrate_supply_params(row):
    ### Use PyPE functions to calibrate remaining supply function parameters.
    if row['IQF']==True:
        paramB = PyPE.CalibrateCosts_monopolyB(row['Total'], row['Total Elasticity'], [0,0], row['Armington'], 
                [row['Armington']], [1.0], [row['share_domestic'],1.0-row['share_domestic']], 
                [1.0,1.0], [0.0,0.0],
                [row['Supply Param A'], 0.0], [row['Value_domestic'],row['Value_import']], [-1,-1], ['asymptote', 'fixed'], 
                form_demand='CES')
    else:
        paramB = PyPE.CalibrateCosts(row['Total'], row['Total Elasticity'], [0,0], row['Armington'], 
                [row['Armington']], [1.0], [row['share_domestic'],1.0-row['share_domestic']], 
                [1.0,1.0], [0.0,0.0],
                [row['Supply Param A'],0.0], [row['Value_domestic'],row['Value_import']], [-1,-1], ['standard', 'fixed'], 
                form_demand='CES')    
    return paramB

def solve_prices(row,simulation_bool):
    # simulation_bool = True if simulation, False if actual
    if simulation_bool == True:
        sfx = 'Simulation Imports'
    else:
        sfx = 'sf0'
    
    # Call equilibrium solving function based on market
    if row['IQF']==True:
        prices = PyPE.SolvePrices_monopolyB(row['Total'], row['Total Elasticity'], [0,0], row['Armington'], 
                [row['Armington']], [1.0], [row['share_domestic'],1.0-row['share_domestic']], 
                [1.0,1.0], [0.0,0.0],
                [row['Supply Param A'], 0.0], [row['sd0'],row[sfx]], [-1,-1], ['asymptote', 'fixed'], 
                form_demand='CES')
    else:
        prices = PyPE.SolvePrices(row['Total'], row['Total Elasticity'], [0,0], row['Armington'], 
                [row['Armington']], [1.0], [row['share_domestic'],1.0-row['share_domestic']], 
                [1.0,1.0], [0.0,0.0],
                [row['Supply Param A'],0.0], [row['sd0'],row[sfx]], [-1,-1], ['standard', 'fixed'], 
                form_demand='CES')
    return prices

def solve_prods(row,simulation_bool):
    # simulation_bool = True if simulation, False if actual
    if simulation_bool == True:
        pdx = 'pd1'
        pfx = 'pf1'
    else:
        pdx = 'pd0'
        pfx = 'pf0'
    
    # First, calculate price indices.
    tmp_Pindex = PyPE.PriceIndex(row['Armington'], [row['share_domestic'],1.0-row['share_domestic']], [row[pdx],row[pfx]],[0,0])
    # Next, calculate quantities. 
    # Note that the same function is used for IQF and Other
    qdx = PyPE.FirmDemandCES(row['Total'], row['Total Elasticity'], row['Armington'], 
                row['Armington'], 1.0, row['share_domestic'], 
                row[pdx], 0.0, tmp_Pindex, tmp_Pindex)
    qfx = PyPE.FirmDemandCES(row['Total'], row['Total Elasticity'], row['Armington'], 
                row['Armington'], 1.0, 1.0-row['share_domestic'], 
                row[pfx], 0.0, tmp_Pindex, tmp_Pindex)
    return [qdx,qfx]

def nearzero_01(df):
    # If the change over actual is negative and small, it is from a rounding error
    col_list = ['pd01','qd01']
    for col in col_list:
        if df[col].dtype=='float64':
            #df.loc[(df[col]<0)&(df[col]>-0.1),col]=0.00
            #df.loc[df[col].abs()<10**(-10**(-15)),col]=0.00
            df.loc[df[col].abs()<0.00000001,col]=0.00
    return df

def change_in_imports(df):
    df['shift_actual'] = df.groupby(['IQF','Armington'])['Value_import'].shift(1)
    df['shift_sim'] = df.groupby(['IQF','Armington'])['Simulation Imports'].shift(1)
    df['Import PC Actual'] = 100*(df['Value_import']-df['shift_actual'])/df['shift_actual']
    df['Import PC Simulation'] = 100*(df['Simulation Imports']-df['shift_sim'])/df['shift_sim']
    return df.drop(['shift_actual','shift_sim'],axis=1)

