# -*- coding: utf-8 -*-
"""
Read me:
This model accompanies the USITC report 'Raspberries for Processing:
Conditions of Competition between U.S. and Foreign Suppliers, with a
Focus on Washington State,' Inv. 332-577. The report includes a quantitative
analysis of the impact of increased imports of raspberries for processing to
the U.S. in recent years. Economic effects are modeled separately for 
Individually Quick Frozen (IQF) and non-IQF raspberries for processing.
For each product, consumers choose between domestic and imported sources.
The model is calibrated to yearly sales between 2010 and 2019. The model is
then used to simulate a scenario were imported raspberries for processing did
not rise as quickl over the period of analysis.

The simulation scenario is based on year-over-year percentage change in the
value of imports to the U.S. For each product, a single set of consecutive 
years are selected as a 'surge' based on having higher growth rates compared 
to the rest of the sample. The non-surge average growth rate is calculated
using the non-surge years. The percentage changes of the surge years are then
reduced by the difference between the surge and the non-surge average. The
resulting sequence of yearly percentage changes has the non-surge average for
the full 2010-2019 period while still allowing year-to-year changes. The
model is then simulated with the no-surge scenario.

The two models share many features: 
-   CES demand between domestic and imports.
-   Imports are aggregated, not differentiated by foreign source.
-   Import supply is assumed to be fixed (perfectly inelastic). The market
    conditions underlying this assumption are discussed in the report.

In the IQF model:
-   Domestic supply has asymptotic marginal costs.
-   Pricing and production is approximated by the monopoly equilibrium.

In the non-IQF model:
-   Domestic supply has a constant price elasticity of supply.
-   Pricing and production are solved by the competitive equilibrium.

This model file requires the data files 'rr_imports.xlsx',
'raspberries_census_revisions_by district.xlsx',
 and '12182020_raspberry_pricing_wrrc.xlsx'
to be installed in the same folder as this file.

This model file requires 'data_processing.py', 'helper.py', 'rr_outputs.py',
and 'pype_perfect.py' to be installed in the same folder as this file. 
Those files contain functions that are called in this file.

"""
#######################
### Import packages ###
#######################
# External
import numpy as np
from scipy import stats
import os


# Internal
import data_processing as dataproc
import helper as hlpr
import rr_outputs as outs

############################
### User Input Variables ###
############################
## IQF Model Settings
surge_years_IQF = [2,3] # 2012-2013, based on staff analysis of year-over-year percent changes
Qmax_multiplier_IQF = 1.1    # IQF maximum capacity (above actual quantity and allow 
                        # supply parameter to be calibrated to match actual quantity).
total_elasticity_IQF = -1.0 # Standard value

## Other (non-IQF) Model Settings
surge_years_Other = [1,2,3,4]   # 2011-2014, based on staff analysis of year-over-year percent changes
supply_elasticity_Other = 1.0 # Standard value
total_elasticity_Other = -1.0 # Standard value

## Algorithm Settings
surge_reduction = 1.0   # Fraction of average surge-over-non-surge growth rate
                        # to be removed in simulation. 1.0 = full removal, so
                        # surge year average matches non-surge average.
n_specifications = 103  # Number of Armington elasticity parameters to use for alternate specifications.
                        # If this variable is 1, only produce a point estimate.

## Plots and Tables Settings
# Set percentile levels for reporting low and high values
hi_percentile = 0.95    # Should be > 0.5 and < 1.0
lo_percentile = 0.05    # Should be > 0.0 and < 0.5

folder_outputs = 'outputs'

###################################################
### Import and clean data using functions in ee ###
###################################################
# Open domestic and import data 2010-2020
df_domestic = dataproc.RRDomesticClean('12182020_raspberry_pricing_wrrc.xlsx')
df_import = dataproc.RRDataWebClean2('rr_imports.xlsx').reset_index()

# Only keep lines for 2010 and later due to domestic data availability.
df_import = df_import.astype({'Year':'int'})
# Divide to millions of dollars
df_agg = dataproc.RRAggregator3(df_import,df_domestic)
df_agg[['Value_import', 'Value_domestic', 'Total']] = df_agg[['Value_import', 'Value_domestic', 'Total']]/1000000


#########################################################
### Variables and lists used in calibration functions ###
#########################################################

# List of years in sample
year_list = list(df_agg.loc[df_agg['Total'].notnull(),'Year'].unique())

# Value imported, used to calculate simulation inputs
Qimports_IQF = list(df_agg.loc[(df_agg['Total'].notnull())&(df_agg['IQF']==True),'Value_import'])
Qimports_Other = list(df_agg.loc[(df_agg['Total'].notnull())&(df_agg['IQF']==False),'Value_import'])

#######################################
### Estimate Armington elasticities ###
#######################################
# Choose IQF or Other subsets of data and call Armington estimator function
tmp = df_import.copy().drop(['cv','ldpv'],axis=1).rename({'cv_IQF':'cv','ldpv_IQF':'ldpv'},axis=1)
arm_IQF_mean, arm_IQF_se = dataproc.ee6(tmp)
tmp = df_import.copy().drop(['cv','ldpv'],axis=1).rename({'cv_Other':'cv','ldpv_Other':'ldpv'},axis=1)
arm_Other_mean, arm_Other_se = dataproc.ee6(tmp)

# Use Armington elasticity estimate and standard error to create distribution of Armington elasticities
# Create linearly spaced list between 0 and 1 (exclusive) then use inverse cdf to approximate normal distribution
zeroone=np.linspace(0,1,n_specifications)[1:-1]
arm_IQF_dist = [1.0-stats.norm.ppf(val, loc=arm_IQF_mean, scale=arm_IQF_se) for val in zeroone]
arm_Other_dist = [1.0-stats.norm.ppf(val, loc=arm_Other_mean, scale=arm_Other_se) for val in zeroone]

#####################################################
### Set up dataframe for model inputs and results ###
#####################################################
# The df dataframe will hold all model inputs and results.
# Each row represents a market (IQF or Other), year, and specification (Armington elasticity).
# Each column represents an input or output variable.
# columns ending in '0' are from data, columns ending in '1' are simulation
# columns with last non-numeric character 'd' are domestic, 'f' are imports

### Start from the dataframe with domestic and import values
df = df_agg.copy()

### Include model parameter columns
df = hlpr.include_supply_param(df,Qmax_multiplier_IQF,supply_elasticity_Other)

### Include the distribution of estimated Armington elasticities and the chosen total elasticities
df = hlpr.include_elasticities(df,arm_IQF_dist,arm_Other_dist,
                               total_elasticity_IQF,total_elasticity_Other)

### Calculate and include simulated imports
df = hlpr.simulation_imports(df,year_list,
                        Qimports_IQF,surge_years_IQF,
                        Qimports_Other,surge_years_Other,surge_reduction)

### Calibrate model to actual data and calculate prices and quantities
df[['sd0','sf0']] = df.apply(lambda row: hlpr.calibrate_supply_params(row), result_type='expand',axis=1)
df[['pd0','pf0']] = df.apply(lambda row: hlpr.solve_prices(row, False), result_type='expand',axis=1)
df[['qd0','qf0']] = df.apply(lambda row: hlpr.solve_prods(row, False), result_type='expand',axis=1)

### Solve simulation scenarios (prices and quantities)
df[['pd1','pf1']] = df.apply(lambda row: hlpr.solve_prices(row, True), result_type='expand',axis=1)
df[['qd1','qf1']] = df.apply(lambda row: hlpr.solve_prods(row, True), result_type='expand',axis=1)

#################################
### Output figures and tables ###
#################################
if not os.path.exists(folder_outputs):
    os.makedirs(folder_outputs)

### Output tables
    
df, df_short = outs.output_tables(df,lo_percentile,hi_percentile,
                                  folder_outputs)

### Plots and visualizations 


outs.generate_figures(df,df_short,year_list,
                     surge_years_IQF,Qimports_IQF,
                     surge_years_Other,Qimports_Other,folder_outputs)

###########################################################
### Import visualizations not directly used in modeling ###
###########################################################
outs.vis_stackplots(df_domestic,df_import,folder_outputs,folder_outputs)
outs.vis_pieplots(df_import,folder_outputs,folder_outputs)