# -*- coding: utf-8 -*-
"""
Read me:
This file supports the main program file. It includes functions
required to calibrate or solvethe equilibrium for perfect competition or
monopoly models.
"""
########################### VARIABLE DESCRIPTIONS #############################
# VARIABLE               LEN     DESCRIPTION
   ## TOTAL MARKET DEMAND ##
# elasticity_total       1       price elasticity of demand for overall market
#                                (negative, =-1 for constant total expenditure)
# shift_demand           1       shifter on overall demand. If elasticity_total
#                                 is -1, this is budget for total expeniture
   ## CES NESTS ##
# m                      1       number of nests
# n                      1       number of firms
# nests                  n       list of 0 and 1 to indicate nest per firm
# elasticity_outer       1       elasticity between nests
# elasticities_inner     m       within nest elasticities, >= outer elasticity 
# weights_outer          m       nest weights
# weights_inner          n       firm weights
   ## PRICES AND TARIFFS ##
# prices                 n       firm prices
# tariffs                n       ad valorem tariffs (0 for no tariff)
      # This next set is not really implemented yet. #
# tariffs_TRQ            n       post-quota tariffs
# quotas_TRQ             n       quota above which post-quota tariff is applied
#                                 set to -1 if no TRQ
   ## SUPPLY FUNCTION ##
# shifts_supply          n       scalar for firm supply functions. Marginal
#                                 cost if elasticity_supply = 1.
# elasticities_supply    n       price elasticity of supply
###############################################################################
import numpy as np
from scipy.optimize import fsolve

### Firm-level Functions ###
# These provide the basis for later functions.

def PriceIndex(elasticity, weights, prices,tariffs='none'):
    # if no tariffs, createvector of zeros.
    if tariffs == 'none':
        tariffs=np.zeros(len(prices))
    # Initialize the price index as 0.0.
    price_index = 0.0    
    # For each price, calculate the inside of the index.
    for i in range(len(prices)):
        price_index = (price_index + 
                       weights[i]*(prices[i]*(1+tariffs[i]))**(1-elasticity))        
    # Finally, return the value after applying exponent.
    return price_index**(1/(1-elasticity))

def FirmDemandCES(shift_demand, elasticity_total, elasticity_outer, 
            elasticity_inner, weight_outer, weight_inner, 
            price, tariff, price_index_outer, price_index_inner):
    return (shift_demand*price_index_outer**elasticity_total
                *weight_outer*weight_inner
                *(price_index_inner/price_index_outer)**(-elasticity_outer)
                *(price*(1.0+tariff)/price_index_inner)
                  **(-elasticity_inner)
                )
def FirmSupply(elasticity, shift, price, quota, form_supply):
    # Different calculation based on form of supply curve
    if form_supply=='standard':
        Qs = shift*price**elasticity
    elif form_supply=='flat':
        # perfect elasticity is 0 if price<marginal cost, otherwise mark
        # it as -1 to be handled in market clearing function.
        if price<shift:
            # in this form, shift represents marginal cost
            Qs = 0.0
        else:
            Qs=-1
    elif form_supply=='asymptote':
        # elasticity and shift don't really work normally here.
        # shift is chosen as the numerator since that is the calibrated param
        Qs = max(0,elasticity-shift/price)
    elif form_supply=='fixed':
        Qs = shift
    # Replace quantity supplied with quota if quantity too high.
    if ((quota>=0) & (quota<Qs)):
        Qs = quota
    return Qs

def FirmMC(elasticity, shift, quantity, quota, form_supply):
    # Mark MC = -1 if vertical MC due to quota
    if ((quota>=0) & (quota<quantity)):
        MC = -1
    # Different calculation based on form of supply curve
    elif (form_supply=='standard') & (elasticity!=0):
            MC = (quantity/shift)**(1/elasticity)
    elif form_supply=='flat':
        MC = shift
    elif form_supply=='asymptote':
        if quantity >= elasticity:
            MC = np.inf
        else:
            MC = shift/(elasticity-quantity) 
    elif (form_supply=='fixed') | ((form_supply=='standard') & (elasticity==0)):
        if quantity<shift:
            MC = 0.0
        elif quantity==shift:
            MC = -1
        else:
            MC = np.inf
    return MC
    

### Equilibrium equations that call the firm-level functions
def ExcessDemand(shift_demand, elasticity_total, nests, elasticity_outer, 
            elasticities_inner, weights_outer, weights_inner, 
            prices, tariffs,
            elasticities_supply, shifts_supply, quotas, forms_supply, 
            form_demand='CES'):
    # Outputs a list which is Qd-Qs for each firm. When the whole list is 0.0,
    #  the market is in a perfectly competitive equilibrium.
    
    ## Reorganize firm information into the nests ##
    nest_max = max(nests)+1

    # Loop over nests and calculate price index for each nest
    Pinners = np.zeros(nest_max)
    for h in range(nest_max):
        sublist = [idx for idx,val in enumerate(nests) if val==h]
        if sublist:   # True if nonempty list
            Pinners[h] = PriceIndex(elasticities_inner[h], 
                                    [weights_inner[i] for i in sublist], 
                                    [prices[i] for i in sublist],
                                    [tariffs[i] for i in sublist])
    Pouter = PriceIndex(elasticity_outer, weights_outer, Pinners)
    
    ## Initialize excess demand
    Qx = [7]*len(nests) # np.zeros(len(nests))
    # Loop over firms to calculate supply and demand
    for i in range(len(nests)):
        h = nests[i]
        Qd = FirmDemandCES(shift_demand, elasticity_total, elasticity_outer, 
            elasticities_inner[h], weights_outer[h], weights_inner[i], 
            prices[i], tariffs[i], Pouter, Pinners[h])
        Qs = FirmSupply(elasticities_supply[i], shifts_supply[i], prices[i], quotas[i], forms_supply[i])
        if Qs == -1:
            Qs = Qd
        Qx[i] = Qd-Qs
    return Qx

def MarginalRevenue(shift_demand, elasticity_total, elasticity_outer, 
            elasticity_inner, weight_outer, weight_inner, 
            price, tariff, price_index_outer, price_index_inner):
    Qd = FirmDemandCES(shift_demand, elasticity_total, elasticity_outer, 
            elasticity_inner, weight_outer, weight_inner, 
            price, tariff, price_index_outer, price_index_inner)
    Pouter_drv = (weight_outer*price_index_inner**(-elasticity_outer)
                  *price_index_outer**elasticity_outer)
    Pinner_drv = (
            weight_inner*(1+tariff)**(1-elasticity_inner)
            *price_index_inner**elasticity_inner
            *price**(-elasticity_inner)
            )
    tmp = (     # This is dQ/dp
                (shift_demand*weight_outer
                     *weight_inner
                     *(1+tariff)**(-elasticity_inner))
                *(price_index_outer**(elasticity_total+elasticity_outer)
                      *price_index_inner
                      **(elasticity_inner-elasticity_outer)
                      *price**(-elasticity_inner))
                *((elasticity_total+elasticity_outer)*price_index_outer**(-1)
                      *Pouter_drv*Pinner_drv
                      +(elasticity_inner-elasticity_outer)
                      *price_index_inner**(-1)*Pinner_drv
                      +(-elasticity_inner)*price**(-1))
                )
    return price+Qd/tmp

def MarginalProfit(shift_demand, elasticity_total, nests, elasticity_outer, 
            elasticities_inner, weights_outer, weights_inner, 
            prices, tariffs,
            elasticities_supply, shifts_supply, quotas, forms_supply, 
            form_demand='CES'):
    # Used for monopoly pricing. Calculate marginal revenue and marginal cost.
    # Profit is maximized at the price where MR = MC.
    
    # Calculate the price index for each nest.
    nest_max = max(nests)+1
    Pinners = np.zeros(nest_max)
    Pouter_drv = np.zeros(nest_max)
    for h in range(nest_max):
        sublist = [idx for idx,val in enumerate(nests) if val==h]
        if sublist:   # True if nonempty list
            Pinners[h] = PriceIndex(elasticities_inner[h], 
                                    [weights_inner[i] for i in sublist], 
                                    [prices[i] for i in sublist],
                                    [tariffs[i] for i in sublist])
            # Calculate the outer derivative (still need to multiply later)
            Pouter_drv[h] = (weights_outer[h]
                                  *Pinners[h]**(-elasticity_outer))
    Pouter = PriceIndex(elasticity_outer, weights_outer, Pinners)
    Pouter_drv = [item*Pouter**elasticity_outer for item in Pouter_drv]
    
    # Loop over firms and calculate marginal profit
    MPr = np.zeros(len(nests))
    for i in range(len(nests)):
        # First, marginal revenue
        Qd = FirmDemandCES(shift_demand, elasticity_total, elasticity_outer, 
            elasticities_inner[h], weights_outer[h], weights_inner[i], 
            prices[i], tariffs[i], Pouter, Pinners[h])
        Pinners_drv = (
            weights_inner[i]*(1+tariffs[i])**(1-elasticities_inner[nests[i]])
            *Pinners[nests[i]]**elasticities_inner[nests[i]]
            *prices[i]**(-elasticities_inner[nests[i]])
            )
        tmp = (     # This is dQ/dp
                (shift_demand*weights_outer[nests[i]]
                     *weights_inner[i]
                     *(1+tariffs[i])**(-elasticities_inner[nests[i]]))
                *(Pouter**(elasticity_total+elasticity_outer)
                      *Pinners[nests[i]]
                      **(elasticities_inner[nests[i]]-elasticity_outer)
                      *prices[i]**(-elasticities_inner[nests[i]]))
                *((elasticity_total+elasticity_outer)*Pouter**(-1)
                      *Pouter_drv[nests[i]]*Pinners_drv
                      +(elasticities_inner[nests[i]]-elasticity_outer)
                      *Pinners[nests[i]]**(-1)*Pinners_drv
                      +(-elasticities_inner[nests[i]])*prices[i]**(-1))
                )
        MR = prices[i]+Qd/tmp
        # Next, marginal cost
        MC = FirmMC(elasticities_supply[i],shifts_supply[i],
                    Qd,quotas[i],forms_supply[i])
        if MC == -1:
            MC = MR
        
        # Finally, marginal profit
        MPr[i] = MR - MC
    
    return MPr

################################################
### Solver functions for perfect competition ###
################################################
def CalibrateCosts(shift_demand, elasticity_total, nests, elasticity_outer, 
            elasticities_inner, weights_outer, weights_inner, 
            prices, tariffs,
            elasticities_supply, shifts_supply_guess, quotas, forms_supply, 
            form_demand='CES'):
    tmp = fsolve(lambda shifts_supply:
                                ExcessDemand(shift_demand, elasticity_total, 
                                             nests, elasticity_outer, 
                                             elasticities_inner, weights_outer, 
                                             weights_inner, prices, tariffs,
                                             elasticities_supply, 
                                             shifts_supply, quotas, 
                                             forms_supply, form_demand='CES'),
                                shifts_supply_guess)
    for i in range(len(tmp)):
        if forms_supply[i]=='flat':
            tmp[i] = 1.0
    return tmp

def SolvePrices(shift_demand, elasticity_total, nests, elasticity_outer, 
            elasticities_inner, weights_outer, weights_inner, 
            prices_guess, tariffs,
            elasticities_supply, shifts_supply, quotas, forms_supply, 
            form_demand='CES'):
    return fsolve(lambda prices:
                                ExcessDemand(shift_demand, elasticity_total, 
                                             nests, elasticity_outer, 
                                             elasticities_inner, weights_outer, 
                                             weights_inner, prices, tariffs,
                                             elasticities_supply, 
                                             shifts_supply, quotas, 
                                             forms_supply, form_demand='CES'),
                                prices_guess)

#####################################
### Solver functions for monopoly ###
#####################################



def MonopolyObj(shift_demand, elasticity_total, nests, elasticity_outer, 
            elasticities_inner, weights_outer, weights_inner, 
            prices, tariffs,
            elasticities_supply, shifts_supply, quotas, forms_supply, 
            form_demand='CES'):
            ### This function primarily minimizes the absolute value of marginal profit
            #   but uses alternate minimization for ineleastic supply. ###
    
    # define the objective function (objective is to make all zero)
    nest_max = max(nests)+1
    Pinners = np.zeros(nest_max)
    for h in range(nest_max):
        sublist = [idx for idx,val in enumerate(nests) if val==h]
        if sublist:   # True if nonempty list
            Pinners[h] = PriceIndex(elasticities_inner[h], 
                                    [weights_inner[i] for i in sublist], 
                                    [prices[i] for i in sublist],
                                    [tariffs[i] for i in sublist])
    Pouter = PriceIndex(elasticity_outer, weights_outer, Pinners)
        
    tmp = np.zeros(len(nests))
    for i in range(len(nests)):
        Qd = FirmDemandCES(shift_demand, elasticity_total, elasticity_outer, 
                               elasticities_inner[nests[i]], weights_outer[nests[i]], weights_inner[i], 
                               prices[i], tariffs[i], Pouter, Pinners[nests[i]])
        if (forms_supply[i]=='fixed') | ((forms_supply[i]=='standard') & (elasticities_supply[i]==0)):
            # Make it so firm has to produce exactly the fixed amount
            # objective just becomes market clearing instead of MR=MC.
            tmp[i] = Qd-shifts_supply[i]
        else:
            MR = MarginalRevenue(shift_demand, elasticity_total, elasticity_outer, 
                                 elasticities_inner[nests[i]], weights_outer[nests[i]], weights_inner[i], 
                                 prices[i], tariffs[i], Pouter, Pinners[nests[i]])
            MC = FirmMC(elasticities_supply[i], shifts_supply[i], Qd, quotas[i], forms_supply[i])
            if MC == -1:
                tmp[i]=0.0
            else:
                tmp[i] = MR-MC
    return tmp

def CalibrateCosts_monopolyB(shift_demand, elasticity_total, nests, elasticity_outer, 
            elasticities_inner, weights_outer, weights_inner, 
            prices, tariffs,
            elasticities_supply, shifts_supply_guess, quotas, forms_supply, 
            form_demand='CES'):    
    return fsolve(lambda shifts_supply:
                  MonopolyObj(shift_demand, elasticity_total, nests, elasticity_outer, 
                              elasticities_inner, weights_outer, weights_inner, 
                              prices, tariffs,
                              elasticities_supply, shifts_supply, quotas, forms_supply, 
                              form_demand='CES'),
                  shifts_supply_guess)

def SolvePrices_monopolyB(shift_demand, elasticity_total, nests, elasticity_outer, 
            elasticities_inner, weights_outer, weights_inner, 
            prices_guess, tariffs,
            elasticities_supply, shifts_supply, quotas, forms_supply, 
            form_demand='CES'):    
    return fsolve(lambda prices:
                  MonopolyObj(shift_demand, elasticity_total, nests, elasticity_outer, 
                              elasticities_inner, weights_outer, weights_inner, 
                              prices, tariffs,
                              elasticities_supply, shifts_supply, quotas, forms_supply, 
                              form_demand='CES'),
                  prices_guess)
        
def SolvePrices_monopoly(shift_demand, elasticity_total, nests, elasticity_outer, 
            elasticities_inner, weights_outer, weights_inner, 
            prices_guess, tariffs,
            elasticities_supply, shifts_supply, quotas, forms_supply, 
            form_demand='CES'):
    return fsolve(lambda prices:
                                MarginalProfit(shift_demand, elasticity_total, 
                                             nests, elasticity_outer, 
                                             elasticities_inner, weights_outer, 
                                             weights_inner, prices, tariffs,
                                             elasticities_supply, 
                                             shifts_supply, quotas, 
                                             forms_supply, form_demand='CES'),
                                prices_guess)
