# -*- coding: utf-8 -*-
"""
Read me:
This file create Excel sheets and png files with various model outputs and 
figures.
"""
import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import matplotlib.lines as mlines
import matplotlib.patches as mpatches
import seaborn as sns

import helper as hlpr

def output_tables(df,lo_percentile,hi_percentile,folder_tables):
    ### Calculate expenditures (price*quantity)
    df['xd0'] = df['pd0']*df['qd0']
    df['xd1'] = df['pd1']*df['qd1']
    df['xf0'] = df['pf0']*df['qf0']
    df['xf1'] = df['pf1']*df['qf1']
    df['x0'] = df['xd0']+df['xf0']
    df['x1'] = df['xd1']+df['xf1']
    
    ### Calculate percentage difference in actual and counterfactual domestic prices and production
    df['pd01'] = 100*(df['pd1']-df['pd0'])/df['pd0']
    df['qd01'] = 100*(df['qd1']-df['qd0'])/df['qd0']
    
    ### Caculate percentage change in actual and simulation import value
    df = hlpr.change_in_imports(df)
    
    ### Values extremely close to 0 are most likely rounding errors
    df = hlpr.nearzero_01(df)
    
    ### Table with point estimates and 10th and 90th percentiles
    df_short = df.groupby(['Year','IQF']).quantile(.5).reset_index()
    df_short['percentile'] = 'mid'
    
    # actually 5th percentile
    df_tmplo = df.groupby(['Year','IQF']).quantile(lo_percentile).reset_index()
    df_tmplo['percentile'] = 'lo'
    df_short = df_short.append(df_tmplo,sort=False)
    
    # actually 95th percentile
    df_tmphi = df.groupby(['Year','IQF']).quantile(hi_percentile).reset_index()
    df_tmphi['percentile'] = 'hi'
    df_short = df_short.append(df_tmphi,sort=False).reset_index(drop=True)
    
    ### Table with production and prices quartiles
    quartiles = [0,.25,.5,.75,1]
    df_pquart = pd.DataFrame()
    df_qquart = pd.DataFrame()
    for quartile in quartiles:
        tmp_p = df[['Year','IQF','pd01']].groupby(['Year','IQF']).quantile(quartile).reset_index()
        tmp_p['quartile']=str(int(quartile*100))
        df_pquart = df_pquart.append(tmp_p,sort=False)
        tmp_q = df[['Year','IQF','qd01']].groupby(['Year','IQF']).quantile(quartile).reset_index()
        tmp_q['quartile']=str(int(quartile*100))
        df_qquart = df_qquart.append(tmp_q,sort=False)
    
    df_pquart = df_pquart.pivot_table(index=['IQF','quartile'],columns='Year',values='pd01').reset_index()
    df_qquart = df_qquart.pivot_table(index=['IQF','quartile'],columns='Year',values='qd01').reset_index()
    
    ######################################################
    ### Write dataframe and various subframes to Excel ###
    ######################################################
    # Tell Python how to write the data to Excel
    writer = pd.ExcelWriter(folder_tables+'/results.xlsx',engine='xlsxwriter')
    
    # Write the full dataframe and a version with only 5th, 50th, 95th percentiles
    df.to_excel(writer, sheet_name='Full',index=False)
    df_short.to_excel(writer, sheet_name='Short',index=False)
    
    # Generate subframes for IQF expenditures
    df_tmpA = df_short.loc[(df_short['IQF']==True)
                          &(df_short['percentile']=='mid'),
                          ['Year','xd0','xf0']]
    df_tmpB = df_short.loc[(df_short['IQF']==True)
                          &(df_short['percentile']=='mid'),
                          ['Year','xd1','xf1']]
    df_tmpC = df_short.loc[(df_short['IQF']==True)
                          &(df_short['percentile']=='lo'),
                          ['Year','xd1']].rename({'xd1':'xd1_lo'},axis=1)
    df_tmpD = df_short.loc[(df_short['IQF']==True)
                          &(df_short['percentile']=='hi'),
                          ['Year','xd1']].rename({'xd1':'xd1_hi'},axis=1)
    df_tmp = df_tmpA.merge(df_tmpB,how='left',on='Year')
    df_tmp = df_tmp.merge(df_tmpC,how='outer',on='Year')
    df_tmp = df_tmp.merge(df_tmpD,how='outer',on='Year')
    df_tmp.to_excel(writer,sheet_name='IQF Expenditure',index=False)
    
    # Generate subframes for non-IQF expenditures
    df_tmpA = df_short.loc[(df_short['IQF']==False)
                          &(df_short['percentile']=='mid'),
                          ['Year','xd0','xf0']]
    df_tmpB = df_short.loc[(df_short['IQF']==False)
                          &(df_short['percentile']=='mid'),
                          ['Year','xd1','xf1']]
    df_tmpC = df_short.loc[(df_short['IQF']==False)
                          &(df_short['percentile']=='lo'),
                          ['Year','xd1']].rename({'xd1':'xd1_lo'},axis=1)
    df_tmpD = df_short.loc[(df_short['IQF']==False)
                          &(df_short['percentile']=='hi'),
                          ['Year','xd1']].rename({'xd1':'xd1_hi'},axis=1)
    df_tmp = df_tmpA.merge(df_tmpB,how='outer',on='Year')
    df_tmp = df_tmp.merge(df_tmpC,how='outer',on='Year')
    df_tmp = df_tmp.merge(df_tmpD,how='outer',on='Year')
    df_tmp.to_excel(writer,sheet_name='non-IQF Expenditure',index=False)
    
    # Generate subframes for non-IQF expenditures
    df_tmp = df_short.loc[(df_short['IQF']==False)
                          &(df_short['percentile']=='mid'),
                          ['Year','xd0','xf0']]
    
    # Generate subframes for IQF production and prices
    df_tmp = df.loc[df['IQF']==True].pivot(index='Armington',
                                           columns='Year',
                                           values='pd1').reset_index().drop(
                                               'Armington',axis=1)
    df_tmp.to_excel(writer,sheet_name='IQF Prices',index=False)
    
    df_tmp = df.loc[df['IQF']==True].pivot(index='Armington',
                                           columns='Year',
                                           values='qd1').reset_index().drop(
        'Armington',axis=1)
    df_tmp.to_excel(writer,sheet_name='IQF Prod',index=False)
    
    # Generate subframes for non-IQF production and prices
    df_tmp = df.loc[df['IQF']==False].pivot(index='Armington',
                                            columns='Year',
                                            values='pd1').reset_index().drop(
                                                'Armington',axis=1)
    df_tmp.to_excel(writer,sheet_name='non-IQF Prices',index=False)
    
    df_tmp = df.loc[df['IQF']==False].pivot(index='Armington',
                                            columns='Year',
                                            values='qd1').reset_index().drop(
                                                'Armington',axis=1)
    df_tmp.to_excel(writer,sheet_name='non-IQF Prod',index=False)
    
    # IQF production and prices by quartile
    df_pquart.to_excel(writer,sheet_name='Price Quartiles',index=False)
    df_qquart.to_excel(writer,sheet_name='Prod Quartiles',index=False)
    
    ### Domestic df Summary Table 
    # Set up weighted averages for price and production changes
    df_short['pd01_wtd'] = df_short['pd01']*df_short['xd0']
    df_short['qd01_wtd'] = df_short['qd01']*df_short['xd0']
    # Results by product
    results_summary_domestic = df_short.loc[(df_short['Year']>=2016),
                                                 ['IQF','percentile','xd0','xd1','x0','x1',
                                                  'pd01_wtd','qd01_wtd']
                                                 ].groupby(['IQF','percentile']).sum().reset_index()
    # Results for total
    results_summary_domestic = results_summary_domestic.append(df_short.loc[(df['Year']>=2016),
                                                 ['percentile','xd0','xd1','x0','x1',
                                                  'pd01_wtd','qd01_wtd']
                                                 ].groupby('percentile').sum().reset_index(),ignore_index=True,sort=False).reset_index(drop=True)
    
    # Finish weighted averages
    results_summary_domestic['pd01_wtd']=results_summary_domestic['pd01_wtd']/results_summary_domestic['xd0']
    results_summary_domestic['qd01_wtd']=results_summary_domestic['qd01_wtd']/results_summary_domestic['xd0']
    
    # Calculate shares
    results_summary_domestic['share0d']=100*results_summary_domestic['xd0']/results_summary_domestic['x0']
    results_summary_domestic['share1d']=100*results_summary_domestic['xd1']/results_summary_domestic['x1']
    
    results_summary_domestic.to_excel(writer,sheet_name='Summary',index=False)
    
    # Save the Excel file
    writer.save()
    return df, df_short

def generate_figures(df,df_short,year_list,
                     surge_years_IQF,Qimports_IQF,
                     surge_years_Other,Qimports_Other,folder_figures):
    # Set plot style parameters
    default_color = [86/255, 86/255, 86/255]
    #default_color = [51/255, 51/255, 51/255]
    plt.rcParams['text.color'] = default_color
    plt.rcParams['axes.labelcolor'] = default_color
    plt.rcParams['xtick.color'] = default_color
    plt.rcParams['ytick.color'] = default_color
    
    plt.rcParams['font.family'] = 'calibri'
    plt.rcParams['font.size'] = 11
    
    plt.rcParams["figure.dpi"] = 300
    
    constant = 255
    color_counter = [237/ constant, 125 / constant, 49 / constant] # orange
    color_actual = [91 / constant, 155 / constant, 213 / constant] # blue
    
    color_domestic = 'Purple'
    color_import = 'Violet'
    
    ### Total Expenditure Plots
    
    ## IQF
    fig_IQF_expend, ax_IQF_expend = plt.subplots(1,2,sharey=True)
    #fig_IQF_expend.suptitle('Expenditures on non-IQF RR ($M)')
    fig_IQF_expend.set_figwidth(6)
    fig_IQF_expend.set_figheight(2)
    ax_IQF_expend[0].set_title('Actual')
    ax_IQF_expend[0].set_ylabel('Millions of USD')
    ax_IQF_expend[0].stackplot(year_list,
                                 df_short.loc[(df_short['IQF']==True)
                                                   &(df_short['percentile']=='mid'),['xd0','xf0']].values.T,
                                 colors=[color_domestic,color_import],labels=['domestic','imports'])
    #ax_IQF_expend[0].legend(['Domestic','Import'],loc='upper left')
    handles, labels = ax_IQF_expend[0].get_legend_handles_labels()
    
    ax_IQF_expend[0].grid(b=True,which='major',axis='y',color='tab:gray',alpha=0.25)
    ax_IQF_expend[0].spines['top'].set_visible(False)
    ax_IQF_expend[0].spines['bottom'].set_visible(False)
    ax_IQF_expend[0].spines['right'].set_visible(False)
    ax_IQF_expend[0].spines['left'].set_visible(False)
    ax_IQF_expend[0].yaxis.set_tick_params(length=0.0)
    
    ax_IQF_expend[1].set_title('Counterfactual')
    ax_IQF_expend[1].stackplot(year_list,
                                 df_short.loc[(df_short['IQF']==True)
                                                   &(df_short['percentile']=='mid'),['xd1','xf1']].values.T,
                                 colors=[color_domestic,color_import],labels=['domestic','imports'])
    
    ax_IQF_expend[1].grid(b=True,which='major',axis='y',color='tab:gray',alpha=0.25)
    ax_IQF_expend[1].spines['top'].set_visible(False)
    ax_IQF_expend[1].spines['bottom'].set_visible(False)
    ax_IQF_expend[1].spines['right'].set_visible(False)
    ax_IQF_expend[1].spines['left'].set_visible(False)
    ax_IQF_expend[1].yaxis.set_tick_params(length=0.0)
    
    ax_IQF_expend[0].legend(handles, labels, loc='lower right',
                          ncol=2,bbox_to_anchor=(1.7,-0.5))
    
    plt.savefig(folder_figures+'/expend_iqf.png',bbox_inches = "tight")
    
    ## Other
    fig_Other_expend, ax_Other_expend = plt.subplots(1,2,sharey=True)
    #fig_Other_expend.suptitle('Expenditures on non-IQF RR ($M)')
    fig_Other_expend.set_figwidth(6)
    fig_Other_expend.set_figheight(2)
    ax_Other_expend[0].set_title('Actual')
    ax_Other_expend[0].set_ylabel('Millions of USD')
    ax_Other_expend[0].stackplot(year_list,
                                 df_short.loc[(df_short['IQF']==False)
                                                   &(df_short['percentile']=='mid'),['xd0','xf0']].values.T,
                                 colors=[color_domestic,color_import],labels=['domestic','imports'])
    handles, labels = ax_Other_expend[0].get_legend_handles_labels()
    
    ax_Other_expend[0].grid(b=True,which='major',axis='y',color='tab:gray',alpha=0.25)
    ax_Other_expend[0].spines['top'].set_visible(False)
    ax_Other_expend[0].spines['bottom'].set_visible(False)
    ax_Other_expend[0].spines['right'].set_visible(False)
    ax_Other_expend[0].spines['left'].set_visible(False)
    ax_Other_expend[0].yaxis.set_tick_params(length=0.0)
    
    ax_Other_expend[1].set_title('Counterfactual')
    ax_Other_expend[1].stackplot(year_list,
                                 df_short.loc[(df_short['IQF']==False)
                                                   &(df_short['percentile']=='mid'),['xd1','xf1']].values.T,
                                 colors=[color_domestic,color_import],labels=['domestic','imports'])
    
    ax_Other_expend[1].grid(b=True,which='major',axis='y',color='tab:gray',alpha=0.25)
    ax_Other_expend[1].spines['top'].set_visible(False)
    ax_Other_expend[1].spines['bottom'].set_visible(False)
    ax_Other_expend[1].spines['right'].set_visible(False)
    ax_Other_expend[1].spines['left'].set_visible(False)
    ax_Other_expend[1].yaxis.set_tick_params(length=0.0)
    
    ax_Other_expend[0].legend(handles, labels, loc='lower right',
                          ncol=2,bbox_to_anchor=(1.7,-0.5))
    
    plt.savefig(folder_figures+'/expend_other.png',bbox_inches = "tight")
    
    ### Imports (percent changes and levels)
    
    ## IQF
    change_actual_IQF = list(df_short.loc[(df_short['IQF']==True)&(df_short['percentile']=='mid')&(df_short['Year']>2010),'Import PC Actual'])
    change_sim_IQF = list(df_short.loc[(df_short['IQF']==True)&(df_short['percentile']=='mid')&(df_short['Year']>2010),'Import PC Simulation'])
    level_sim_IQF = list(df_short.loc[(df_short['IQF']==True)&(df_short['percentile']=='mid'),'Simulation Imports'])
    
    fig_IQF_sim, ax_IQF_sim = plt.subplots(nrows=1,ncols=2, sharex=True)
    fig_IQF_sim.set_figwidth(6)
    fig_IQF_sim.set_figheight(2.5)
    
    # First, percentage changes
    ax_IQF_sim[0].set_ylabel('Percent change over previous year')
    ax_IQF_sim[0].plot(year_list[1:],change_sim_IQF,color=color_counter)
    ax_IQF_sim[0].plot(year_list[1:],change_actual_IQF,color=color_actual)
    for i in surge_years_IQF:
        ax_IQF_sim[0].arrow(year_list[i+1],change_actual_IQF[i],0,change_sim_IQF[i]-change_actual_IQF[i],color='green',head_length=2.0,head_width=0.25,length_includes_head=True)
    ax_IQF_sim[0].grid(b=True,which='major',axis='y',color='tab:gray',alpha=0.25)
    ax_IQF_sim[0].spines['top'].set_visible(False)
    ax_IQF_sim[0].spines['bottom'].set_visible(False)
    ax_IQF_sim[0].spines['right'].set_visible(False)
    ax_IQF_sim[0].spines['left'].set_visible(False)
    ax_IQF_sim[0].yaxis.set_tick_params(length=0.0)
    # Now, levels
    ax_IQF_sim[1].set_ylabel('Millions of USD')
    ax_IQF_sim[1].plot(year_list,level_sim_IQF,color=color_counter,label='Simulation')
    ax_IQF_sim[1].plot(year_list,Qimports_IQF,color=color_actual,label='Actual')
    ax_IQF_sim[1].grid(b=True,which='major',axis='y',color='tab:gray',alpha=0.25)
    ax_IQF_sim[1].spines['top'].set_visible(False)
    ax_IQF_sim[1].spines['bottom'].set_visible(False)
    ax_IQF_sim[1].spines['right'].set_visible(False)
    ax_IQF_sim[1].spines['left'].set_visible(False)
    ax_IQF_sim[1].yaxis.set_tick_params(length=0.0)
    
    line_actual = mlines.Line2D([], [], color=color_actual, label='Actual')
    line_counter = mlines.Line2D([], [], color=color_counter, label='Counterfactual')
    ax_IQF_sim[0].legend(handles=[line_actual,line_counter], loc='lower center',
                          ncol=2,bbox_to_anchor=(1.0,-0.5))
    
    plt.savefig(folder_figures+'/actsim_iqf.png',bbox_inches = "tight")
    
    ## Other
    change_actual_Other = list(df_short.loc[(df_short['IQF']==False)&(df_short['percentile']=='mid')&(df_short['Year']>2010),'Import PC Actual'])
    change_sim_Other = list(df_short.loc[(df_short['IQF']==False)&(df_short['percentile']=='mid')&(df_short['Year']>2010),'Import PC Simulation'])
    level_sim_Other = list(df_short.loc[(df_short['IQF']==False)&(df_short['percentile']=='mid'),'Simulation Imports'])
    
    fig_Other_sim, ax_Other_sim = plt.subplots(nrows=1,ncols=2, sharex=True)
    fig_Other_sim.set_figwidth(6)
    fig_Other_sim.set_figheight(2.5)
    
    # First, percentage changes
    ax_Other_sim[0].set_ylabel('Percent change over previous year')
    ax_Other_sim[0].plot(year_list[1:],change_sim_Other,color=color_counter)
    ax_Other_sim[0].plot(year_list[1:],change_actual_Other,color=color_actual)
    for i in surge_years_Other:
        ax_Other_sim[0].arrow(year_list[i+1],change_actual_Other[i],0,change_sim_Other[i]-change_actual_Other[i],color='green',head_length=2.0,head_width=0.25,length_includes_head=True)
    ax_Other_sim[0].grid(b=True,which='major',axis='y',color='tab:gray',alpha=0.25)
    ax_Other_sim[0].spines['top'].set_visible(False)
    ax_Other_sim[0].spines['bottom'].set_visible(False)
    ax_Other_sim[0].spines['right'].set_visible(False)
    ax_Other_sim[0].spines['left'].set_visible(False)
    ax_Other_sim[0].yaxis.set_tick_params(length=0.0)
    # Now, levels
    ax_Other_sim[1].set_ylabel('Millions of USD')
    ax_Other_sim[1].plot(year_list,level_sim_Other,color=color_counter,label='Simulation')
    ax_Other_sim[1].plot(year_list,Qimports_Other,color=color_actual,label='Actual')
    ax_Other_sim[1].grid(b=True,which='major',axis='y',color='tab:gray',alpha=0.25)
    ax_Other_sim[1].spines['top'].set_visible(False)
    ax_Other_sim[1].spines['bottom'].set_visible(False)
    ax_Other_sim[1].spines['right'].set_visible(False)
    ax_Other_sim[1].spines['left'].set_visible(False)
    ax_Other_sim[1].yaxis.set_tick_params(length=0.0)
    
    line_actual = mlines.Line2D([], [], color=color_actual, label='Actual')
    line_counter = mlines.Line2D([], [], color=color_counter, label='Counterfactual')
    ax_Other_sim[0].legend(handles=[line_actual,line_counter], loc='lower center',
                          ncol=2,bbox_to_anchor=(1.0,-0.5))
    
    plt.savefig(folder_figures+'/actsim_other.png',bbox_inches = "tight")
    
    ### Production and prices as subplots
    
    ### Production and prices as box and whiskers subplots
    ## IQF
    fig_IQF_prodprice2, ax_IQF_prodprice2 = plt.subplots(1,2,sharey=True,sharex=True)
    fig_IQF_prodprice2.set_figwidth(6)
    fig_IQF_prodprice2.set_figheight(2.5)
    
    ax_IQF_prodprice2[0].set_title('Production')
    sns.boxplot(ax=ax_IQF_prodprice2[0], data=df.loc[df['IQF']==True], 
                x='Year',y='qd01',
                color=color_counter, whis=[0, 100])
    ax_IQF_prodprice2[0].set_ylabel('Percent over actual')
    ax_IQF_prodprice2[0].set_xlabel('')
    ax_IQF_prodprice2[0].set_ylim(top=40)
    ax_IQF_prodprice2[0].grid(b=True,which='major',axis='y',color='tab:gray',alpha=0.25)
    ax_IQF_prodprice2[0].spines['top'].set_visible(False)
    ax_IQF_prodprice2[0].spines['bottom'].set_visible(False)
    ax_IQF_prodprice2[0].spines['right'].set_visible(False)
    ax_IQF_prodprice2[0].spines['left'].set_visible(False)
    ax_IQF_prodprice2[0].yaxis.set_tick_params(length=0.0)
    ax_IQF_prodprice2[0].set_xticks([0,5,10])  
    
    ax_IQF_prodprice2[1].set_title('Prices')
    sns.boxplot(ax=ax_IQF_prodprice2[1], data=df.loc[df['IQF']==True], 
                x='Year',y='pd01',
                color=color_counter, whis=[0, 100])
    ax_IQF_prodprice2[1].set_ylabel('')
    ax_IQF_prodprice2[1].set_xlabel('')
    ax_IQF_prodprice2[1].grid(b=True,which='major',axis='y',color='tab:gray',alpha=0.25)
    ax_IQF_prodprice2[1].spines['top'].set_visible(False)
    ax_IQF_prodprice2[1].spines['bottom'].set_visible(False)
    ax_IQF_prodprice2[1].spines['right'].set_visible(False)
    ax_IQF_prodprice2[1].spines['left'].set_visible(False)
    ax_IQF_prodprice2[1].yaxis.set_tick_params(length=0.0)
    ax_IQF_prodprice2[1].set_xticks([0,5,10])
    ax_IQF_prodprice2[1].yaxis.set_tick_params(labelleft=True)
    
    plt.savefig(folder_figures+'/boxes_iqf.png',bbox_inches = "tight")
    
    ## Other
    fig_Other_prodprice2, ax_Other_prodprice2 = plt.subplots(1,2,sharey=True,sharex=True)
    fig_Other_prodprice2.set_figwidth(6)
    fig_Other_prodprice2.set_figheight(2.5)
    
    ax_Other_prodprice2[0].set_title('Production')
    sns.boxplot(ax=ax_Other_prodprice2[0], data=df.loc[df['IQF']==False], 
                x='Year',y='qd01',
                color=color_counter, whis=[0, 100])
    ax_Other_prodprice2[0].set_ylabel('Percent over actual')
    ax_Other_prodprice2[0].set_xlabel('')
    ax_Other_prodprice2[0].grid(b=True,which='major',axis='y',color='tab:gray',alpha=0.25)
    ax_Other_prodprice2[0].spines['top'].set_visible(False)
    ax_Other_prodprice2[0].spines['bottom'].set_visible(False)
    ax_Other_prodprice2[0].spines['right'].set_visible(False)
    ax_Other_prodprice2[0].spines['left'].set_visible(False)
    ax_Other_prodprice2[0].yaxis.set_tick_params(length=0.0)
    ax_Other_prodprice2[0].set_xticks([0,5,10])  
    
    ax_Other_prodprice2[1].set_title('Prices')
    sns.boxplot(ax=ax_Other_prodprice2[1], data=df.loc[df['IQF']==False], 
                x='Year',y='pd01',
                color=color_counter, whis=[0, 100])
    ax_Other_prodprice2[1].set_ylabel('')
    ax_Other_prodprice2[1].set_xlabel('')
    ax_Other_prodprice2[1].grid(b=True,which='major',axis='y',color='tab:gray',alpha=0.25)
    ax_Other_prodprice2[1].spines['top'].set_visible(False)
    ax_Other_prodprice2[1].spines['bottom'].set_visible(False)
    ax_Other_prodprice2[1].spines['right'].set_visible(False)
    ax_Other_prodprice2[1].spines['left'].set_visible(False)
    ax_Other_prodprice2[1].yaxis.set_tick_params(length=0.0)
    ax_Other_prodprice2[1].set_xticks([0,5,10])
    ax_Other_prodprice2[1].yaxis.set_tick_params(labelleft=True)
    
    plt.savefig(folder_figures+'/boxes_other.png',bbox_inches = "tight")
    
    plt.close('all')
    
    return

def vis_stackplots(df_domestic,df_import,folder_tables,folder_figures):

    # Create df with both domestic and imports
    df_domestic['Country'] = 'Washington State'
    df_domestic.rename({'Type':'HTS','Revenue':'ldpv'},axis=1,inplace=True)
    df_domestic['ldpv_IQF']=0
    df_domestic['ldpv_Other']=0
    df_domestic.loc[df_domestic['HTS']=='IQF',['ldpv_IQF']] = df_domestic['ldpv']
    df_domestic.loc[df_domestic['HTS']!='IQF',['ldpv_Other']] = df_domestic['ldpv']
    
    df_all = df_import.append(df_domestic,sort=False)[['Year','Country','ldpv','ldpv_IQF','ldpv_Other']].groupby(['Year','Country']).agg('sum').reset_index()
    df_all['ldpv_agg']=df_all['ldpv_IQF']+df_all['ldpv_Other']
    
    # Also convert this dataframe to millions of dollars
    for col in ['ldpv','ldpv_IQF','ldpv_Other','ldpv_agg']:
        df_all[col]=df_all[col]/1000000
    
    ################################
    ### Set some figure defaults ###
    ################################
    default_color = [86/255, 86/255, 86/255]
    #default_color = [51/255, 51/255, 51/255]
    plt.rcParams['text.color'] = default_color
    plt.rcParams['axes.labelcolor'] = default_color
    plt.rcParams['xtick.color'] = default_color
    plt.rcParams['ytick.color'] = default_color
    
    plt.rcParams['font.family'] = 'calibri'
    plt.rcParams['font.size'] = 11
    
    plt.rcParams["figure.dpi"] = 200
    
    ##########################
    ### Stacked Area Plots ###
    ##########################
    
    stackwriter = pd.ExcelWriter(folder_tables+'/stackdata.xlsx',engine='xlsxwriter')
    
    ## IQF RR for processing imports
    top5_IQF = df_all.loc[(df_all['Country']!='Washington State')&(df_all['Year']==2020),['Country','ldpv_IQF']].sort_values('ldpv_IQF',ascending=False)['Country'].head(5).to_list()
    
    df_all_IQF = df_all.loc[(df_all['Country'].isin(top5_IQF))|(df_all['Country']=='Washington State'),['Year','Country','ldpv_IQF']].groupby(['Year','Country']).agg('sum').reset_index()
    tmp = df_all.loc[(df_all['Country'].isin(top5_IQF)==False)&(df_all['Country']!='Washington State'),['Year','ldpv_IQF']].groupby('Year').agg('sum').reset_index()
    tmp['Country']='All others'
    df_all_IQF = df_all_IQF.append(tmp,sort=False)
    # pivot table, but then aggregate countries not in the top 5
    df_all_IQF = df_all_IQF.pivot(index='Year',columns='Country',values='ldpv_IQF')
    # reorder for nice plots
    tmplist = ['Washington State']+top5_IQF+['All others']
    df_all_IQF = df_all_IQF[tmplist].reset_index()
    df_all_IQF.to_excel(stackwriter,sheet_name='IQF',index=False)
    
    fig_stackIQF, ax_stackIQF = plt.subplots()
    fig_stackIQF.set_figwidth(6)
    fig_stackIQF.set_figheight(2.5)
    ax_stackIQF.stackplot(df_all_IQF['Year'],df_all_IQF.drop('Year',axis=1).T,
                          cmap='tab10',labels=tmplist)
    ax_stackIQF.set_ylabel('Millions of USD')
    handles, labels = ax_stackIQF.get_legend_handles_labels()
    ax_stackIQF.legend(handles[::-1], labels[::-1], 
                       loc='upper left', bbox_to_anchor= (1.0, 1.0), ncol=1,
                borderaxespad=0, frameon=True)
    ax_stackIQF.grid(b=True,which='major',axis='y',color='tab:gray',alpha=0.25)
    ax_stackIQF.spines['top'].set_visible(False)
    ax_stackIQF.spines['bottom'].set_visible(False)
    ax_stackIQF.spines['right'].set_visible(False)
    ax_stackIQF.spines['left'].set_visible(False)
    ax_stackIQF.yaxis.set_tick_params(length=0.0)
    #plt.title('Value of IQF raspberries for processing (USD)')
    
    plt.savefig(folder_figures+'/stacked_iqf.png',bbox_inches = "tight")
    
    ## Other RR for processing imports
    top5_Other = df_all.loc[(df_all['Country']!='Washington State')&(df_all['Year']==2020),['Country','ldpv_Other']].sort_values('ldpv_Other',ascending=False)['Country'].head(5).to_list()
    
    df_all_Other = df_all.loc[(df_all['Country'].isin(top5_Other))|(df_all['Country']=='Washington State'),['Year','Country','ldpv_Other']].groupby(['Year','Country']).agg('sum').reset_index()
    tmp = df_all.loc[(df_all['Country'].isin(top5_Other)==False)&(df_all['Country']!='Washington State'),['Year','ldpv_Other']].groupby('Year').agg('sum').reset_index()
    tmp['Country']='All others'
    df_all_Other = df_all_Other.append(tmp,sort=False)
    # pivot table, but then aggregate countries not in the top 5
    df_all_Other = df_all_Other.pivot(index='Year',columns='Country',values='ldpv_Other')
    # reorder for nice plots
    tmplist = ['Washington State']+top5_Other+['All others']
    df_all_Other = df_all_Other[tmplist].reset_index()
    df_all_Other.to_excel(stackwriter,sheet_name='Non-IQF',index=False)
    
    fig_stackOther, ax_stackOther = plt.subplots()
    fig_stackOther.set_figwidth(6)
    fig_stackOther.set_figheight(2.5)
    ax_stackOther.stackplot(df_all_Other['Year'],df_all_Other.drop('Year',axis=1).T,
                          cmap='tab10',labels=tmplist)
    ax_stackOther.set_ylabel('Millions of USD')
    handles, labels = ax_stackOther.get_legend_handles_labels()
    ax_stackOther.legend(handles[::-1], labels[::-1], 
                       loc='upper left', bbox_to_anchor= (1.0, 1.0), ncol=1,
                borderaxespad=0, frameon=True)
    ax_stackOther.grid(b=True,which='major',axis='y',color='tab:gray',alpha=0.25)
    ax_stackOther.spines['top'].set_visible(False)
    ax_stackOther.spines['bottom'].set_visible(False)
    ax_stackOther.spines['right'].set_visible(False)
    ax_stackOther.spines['left'].set_visible(False)
    ax_stackOther.yaxis.set_tick_params(length=0.0)
    #plt.title('Value of non-IQF raspberries for processing (USD)')
    
    plt.savefig(folder_figures+'/stacked_other.png',bbox_inches = "tight")
    
    plt.close('all')
    stackwriter.save()
    return

##############################
### Break-outs by HTS Code ###
##############################    
def vis_pieplots(df_import,folder_tables,folder_figures):

    ### Set some figure defaults ###

    default_color = [86/255, 86/255, 86/255]
    #default_color = [51/255, 51/255, 51/255]
    plt.rcParams['text.color'] = default_color
    plt.rcParams['axes.labelcolor'] = default_color
    plt.rcParams['xtick.color'] = default_color
    plt.rcParams['ytick.color'] = default_color
    
    plt.rcParams['font.family'] = 'calibri'
    plt.rcParams['font.size'] = 11
    
    plt.rcParams["figure.dpi"] = 200
    
    labels = ['IQF','Non-IQF','Out of scope']
    color_list = ['tab:blue','tab:orange','tab:green']
    
    handle_list = ['']*len(labels)
    for i in range(len(labels)):
        handle_list[i] = mpatches.Patch(color=color_list[i],label=labels[i])
    
    ### Define product groups ###
    HTS_list = list(df_import['HTS'].unique())
    HTS_list_fresh = ['0810201020','0810209020', # partial
                     '0810201024','0810209024', # in
                     '0810201022','0810209022'] # out
    HTS_list_mixed = ['0811908080','0811908085','0811908095']
    HTS_list_single = ['0811202025','0811202035']
    HTS_list_other = ['2008992120','2007996510','2009896055','2009897055']
    HTS_list_related = ['2007990500','2007996520','2009897065']
    
    ### Reformat dataframe ###
    df_import_wide = df_import[['Year','HTS','Description','ldpv']].groupby(['Year','HTS','Description']).agg('sum').reset_index()
    df_import_wide = df_import_wide.pivot_table(index=['HTS','Description'],columns='Year',values='ldpv')/1000000
    df_import_wide = df_import_wide.reset_index()
    df_import_wide['Group']=''
    #df_import_wide.to_excel('imports_by_HTS-Year.xlsx')
    
    # Add group to df_import_wide
    df_import_wide.loc[df_import_wide['HTS'].isin(HTS_list_fresh),'Group']='Fresh'
    df_import_wide.loc[df_import_wide['HTS'].isin(HTS_list_mixed),'Group']='Mixed'
    df_import_wide.loc[df_import_wide['HTS'].isin(HTS_list_single),'Group']='Single'
    df_import_wide.loc[df_import_wide['HTS'].isin(HTS_list_other),'Group']='Other'
    df_import_wide.loc[df_import_wide['HTS'].isin(HTS_list_related),'Group']='Related'
    df_import_wide.to_excel(folder_tables+'/imports_by_hts_year.xlsx',index=False)
    
    ### Set up Excel sheet for pie charts ###
    piewriter = pd.ExcelWriter(folder_tables+'/raspberry_pies.xlsx',engine='xlsxwriter')
    
    ## Bulk fresh
    figure_fresh,axes_fresh = plt.subplots(2,3,figsize=(6,2.5))
    #figure_fresh.suptitle('Fresh Raspberries')
    tmp_pies=[]
    for i,HTS in enumerate(HTS_list_fresh):
        irow = int(np.floor(i/3))
        icol = i%3
        tmp = list(df_import.loc[df_import['HTS']==HTS,['ldpv_IQF','ldpv_Other','ldpv_OOS']].sum())
        tmp_pies.append([HTS[0:4]+'.'+HTS[4:6]+'.'+HTS[6:]]+tmp)
        tmp1 = [tmp[i] for i in range(len(tmp)) if tmp[i]>0]
        tmp2 = [color_list[i] for i in range(len(tmp)) if tmp[i]>0]
        axes_fresh[irow,icol].pie(tmp1, colors=tmp2,
                                  radius=1,
                                  wedgeprops=dict(width=1,edgecolor='w'))
        axes_fresh[irow,icol].set_xlabel(HTS[0:4]+'.'+HTS[4:6]+'.'+HTS[6:])
    axes_fresh[1,1].legend(handles=handle_list,loc='lower center',ncol=3,bbox_to_anchor=(0.5,-1.0))
    plt.savefig(folder_figures+'/pie_fresh.png',bbox_inches = "tight")
    
    pd.DataFrame(data=tmp_pies,columns=['HTS','IQF','Non-IQF','OOS']).to_excel(
        piewriter, sheet_name='Fresh',index=False)
    
    ## Mixed Bags
    figure_mixed,axes_mixed = plt.subplots(1,3,figsize=(6,2.5))
    #figure_mixed.suptitle('Frozen Mixed Fruit')
    tmp_pies=[]
    for i,HTS in enumerate(HTS_list_mixed):
        irow = 0
        icol = i
        tmp = list(df_import.loc[df_import['HTS']==HTS,['ldpv_IQF','ldpv_Other','ldpv_OOS']].sum())
        tmp_pies.append([HTS[0:4]+'.'+HTS[4:6]+'.'+HTS[6:]]+tmp)
        tmp1 = [tmp[i] for i in range(len(tmp)) if tmp[i]>0]
        tmp2 = [color_list[i] for i in range(len(tmp)) if tmp[i]>0]
        axes_mixed[icol].pie(tmp1, colors=tmp2,
                                  radius=1,
                                  wedgeprops=dict(width=1,edgecolor='w'))
        axes_mixed[icol].set_xlabel(HTS[0:4]+'.'+HTS[4:6]+'.'+HTS[6:])
    axes_mixed[1].legend(handles=handle_list,loc='lower center',
                        ncol=3,bbox_to_anchor=(0.5,-0.5))
    plt.savefig(folder_figures+'/pie_mixed.png',bbox_inches = "tight")
    
    pd.DataFrame(data=tmp_pies,columns=['HTS','IQF','Non-IQF','OOS']).to_excel(
        piewriter, sheet_name='Mixed',index=False)
    
    ## Single Frozen
    figure_single,axes_single = plt.subplots(1,2,figsize=(4,2.5))
    #figure_single.suptitle('Single Product Frozen')
    tmp_pies=[]
    for i,HTS in enumerate(HTS_list_single):
        irow = 0
        icol = i
        tmp = list(df_import.loc[df_import['HTS']==HTS,['ldpv_IQF','ldpv_Other','ldpv_OOS']].sum())
        tmp_pies.append([HTS[0:4]+'.'+HTS[4:6]+'.'+HTS[6:]]+tmp)
        tmp1 = [tmp[i] for i in range(len(tmp)) if tmp[i]>0]
        tmp2 = [color_list[i] for i in range(len(tmp)) if tmp[i]>0]
        axes_single[icol].pie(tmp1, colors=tmp2,
                                  radius=1,
                                  wedgeprops=dict(width=1,edgecolor='w'))
        axes_single[icol].set_xlabel(HTS[0:4]+'.'+HTS[4:6]+'.'+HTS[6:])
    axes_single[0].legend(handles=handle_list,loc='lower right',
                         ncol=3,bbox_to_anchor=(2.5,-0.5))
    plt.savefig(folder_figures+'/pie_single.png',bbox_inches = "tight")
    
    pd.DataFrame(data=tmp_pies,columns=['HTS','IQF','Non-IQF','OOS']).to_excel(
        piewriter, sheet_name='Single',index=False)
    
    
    ## Other Preparations
    figure_other,axes_other = plt.subplots(2,2,figsize=(4,2.5))
    #figure_other.suptitle('Cooked, Juices, and Other Preparations')
    tmp_pies=[]
    for i,HTS in enumerate(HTS_list_other):
        irow = int(np.floor(i/2))
        icol = i%2
        tmp = list(df_import.loc[df_import['HTS']==HTS,['ldpv_IQF','ldpv_Other','ldpv_OOS']].sum())
        tmp_pies.append([HTS[0:4]+'.'+HTS[4:6]+'.'+HTS[6:]]+tmp)
        tmp1 = [tmp[i] for i in range(len(tmp)) if tmp[i]>0]
        tmp2 = [color_list[i] for i in range(len(tmp)) if tmp[i]>0]
        axes_other[irow,icol].pie(tmp1, colors=tmp2,
                                  radius=1,
                                  wedgeprops=dict(width=1,edgecolor='w'))
        axes_other[irow,icol].set_xlabel(HTS[0:4]+'.'+HTS[4:6]+'.'+HTS[6:])
    axes_other[1,0].legend(handles=handle_list,loc='lower left',
                        ncol=3,bbox_to_anchor=(-0.5,-1.0))
    plt.savefig(folder_figures+'/pie_other.png',bbox_inches = "tight",bbox_add_artists=[figure_other.legend])
    
    pd.DataFrame(data=tmp_pies,columns=['HTS','IQF','Non-IQF','OOS']).to_excel(
        piewriter, sheet_name='Other',index=False)
    
    ## Related Products
    figure_related,axes_related = plt.subplots(1,3,figsize=(6,2.5))
    #figure_related.suptitle('Related Products')
    tmp_pies=[]
    for i,HTS in enumerate(HTS_list_related):
        irow = 0
        icol = i
        tmp = list(df_import.loc[df_import['HTS']==HTS,['ldpv_IQF','ldpv_Other','ldpv_OOS']].sum())
        tmp_pies.append([HTS[0:4]+'.'+HTS[4:6]+'.'+HTS[6:]]+tmp)
        tmp1 = [tmp[i] for i in range(len(tmp)) if tmp[i]>0]
        tmp2 = [color_list[i] for i in range(len(tmp)) if tmp[i]>0]
        axes_related[icol].pie(tmp1, colors=tmp2,
                                  radius=1,
                                  wedgeprops=dict(width=1,edgecolor='w'))
        axes_related[icol].set_xlabel(HTS[0:4]+'.'+HTS[4:6]+'.'+HTS[6:])
    axes_related[1].legend(handles=handle_list,loc='lower center',
                          ncol=3,bbox_to_anchor=(0.5,-0.5))
    plt.savefig(folder_figures+'/pie_related.png',bbox_inches = "tight")
    
    pd.DataFrame(data=tmp_pies,columns=['HTS','IQF','Non-IQF','OOS']).to_excel(
        piewriter, sheet_name='Related',index=False)
    
    ############################
    ### Top 5 sources by HTS ###
    ############################
    
    df_topCountries = pd.DataFrame(index=range(5))
    ## Table of top 5 countries for each product code
    for HTS in HTS_list:
        tmp = df_import.loc[df_import['HTS']==HTS,['Country','ldpv']].groupby(['Country']).agg('sum').sort_values('ldpv',ascending=False).reset_index().head(5)
        tmp.rename({'Country':HTS},axis=1,inplace=True)
        tmp.drop('ldpv',axis=1,inplace=True)
        df_topCountries = df_topCountries.merge(tmp,left_index=True,right_index=True,how='left')
    df_topCountries.to_excel(folder_tables+'/top_countries.xlsx')    
    
    ##############################
    ### Make summary pie chart ###
    ##############################
    
    default_color = [51/255, 51/255, 51/255]
    plt.rcParams['text.color'] = default_color
    plt.rcParams['axes.labelcolor'] = default_color
    
    # First, create column for category.
    df_import['category'] = ''
    df_import.loc[df_import['HTS'].isin(HTS_list_fresh),'category']='Fresh for processing'
    df_import.loc[df_import['HTS'].isin(HTS_list_mixed),'category']='Frozen mixed fruit'
    df_import.loc[df_import['HTS'].isin(HTS_list_single),'category']='Single product frozen'
    df_import.loc[df_import['HTS'].isin(HTS_list_other),'category']='Cooked or other preparations'
    df_import.loc[df_import['HTS'].isin(['2009896055','2009897055']),'category']='Juice'
    df_import.loc[df_import['HTS'].isin(HTS_list_related),'category']='Related products'
    
    df_piesum = df_import.loc[df_import['Year']==2020,['HTS','category','ldpv_IQF','ldpv_Other']].groupby(['HTS','category']).agg('sum').reset_index()
    df_piesum = df_piesum.loc[(df_piesum['ldpv_IQF']+df_piesum['ldpv_Other']>0)]
    df_piesum = df_piesum.drop('HTS',axis=1)
    
    df_piesum = df_piesum.groupby('category').agg('sum').reset_index()
    
    
    # Wide to long
    df_piesum = df_piesum.melt(id_vars='category')
    df_piesum['variable']=df_piesum['variable'].str.replace('ldpv_','')
    
    # differentiate IQF and non-IQF single product frozen
    df_piesum.loc[(df_piesum['variable']=='Other')&
                  (df_piesum['category']=='Single product frozen'),'category']='Single product frozen, non-IQF'
    df_piesum.loc[(df_piesum['variable']=='IQF')&
                  (df_piesum['category']=='Single product frozen'),'category']='Single product frozen, IQF'
    # add superscripts
    df_piesum.loc[df_piesum['category']=='Frozen mixed fruit','category']='Frozen mixed fruit$\mathregular{^a}$'
    df_piesum.loc[df_piesum['category']=='Juice','category']='Juice$\mathregular{^b}$'
    
    
    # prep pie chart inputs
    df_piesum = df_piesum[df_piesum['value']>0].sort_values(by=['variable','value'],ascending=[True,False]).reset_index(drop=True)
    piesum_labels = list(df_piesum['category'])
    piesum_IQF = list(df_piesum.loc[df_piesum['variable']=='IQF','value'])
    piesum_Other = list(df_piesum.loc[df_piesum['variable']!='IQF','value'])
    
    piesum_all = piesum_IQF.copy()
    piesum_all.extend(piesum_Other)
    piesum_agg = [sum(piesum_IQF), sum(piesum_Other)]
    
    cmap_agg = plt.get_cmap('tab10')
    colors_agg = cmap_agg([0,1])
    
    cmap_all = plt.get_cmap('tab20c')
    colors_all = cmap_all([0,1,4,5,6,7])
    
    # generate the chart
    fig_pie2, ax_pie2 = plt.subplots(figsize=(4,4))
    size=1
    # outer chart has specific categories
    ax_pie2.pie(piesum_all, radius=2,colors=colors_all,
                wedgeprops=dict(width=size,edgecolor='w'),
                autopct='%1.1f%%',pctdistance=1.10)
    ax_pie2.legend(labels=piesum_labels,bbox_to_anchor=(0.5,-0.55),ncol=3,loc='lower center')
    # inner chart has general groups
    ax_pie2.pie(piesum_agg, radius=2-size,colors=colors_agg,
                wedgeprops=dict(width=size,edgecolor='w'),autopct='%1.1f%%',
                labels=['IQF','Non-IQF'],labeldistance=0.15,
                textprops={'fontsize': 14,'color':'w'})
    plt.savefig(folder_figures+'/pie_all.png',bbox_inches = "tight")
    
    # Turn plotting back on
    plt.close('all')
    
    return